/** ============================================================================
 *  @file   da8xxgem_phy_shmem.c
 *
 *  @path   $(DSPLINK)/gpp/src/arch/DA8XXGEM/shmem/WinCE/
 *
 *  @desc   WinCE shmem driver interface file.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


 /*  ----------------------------------- OS Headers                  */
#include <windows.h>
#include <ceddk.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- Hardware Abstraction Layer  */
#include <da8xxgem_hal.h>
#include <da8xxgem_hal_boot.h>
#include <da8xxgem_hal_intgen.h>
#include <da8xxgem_hal_pwr.h>
#include <da8xxgem_phy_shmem.h>


#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_ARCH_SHMEM_PHY

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON TRC_SetReason (status,FID_C_ARCH_PHY_SHMEM,__LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/*  ============================================================================
 *  @const  CHIP_CFG_BASE
 *
 *  @desc   Base address of CFG memory
 *  ============================================================================
 */
#define CHIP_CFG_BASE               0x01C00000

/*  ============================================================================
 *  @const  CHIP_CFG_MAP_SIZE
 *
 *  @desc   Size of the CFG memory
 *  ============================================================================
 */
#define CHIP_CFG_MAP_SIZE           0x1000000

/*  ============================================================================
 *  @const  OFFSET_SYSTEM_MODULE
 *
 *  @desc   Offset of the system module registers from the base of the CFG
 *          memory.
 *  ============================================================================
 */
#define  OFFSET_SYSTEM_MODULE       0x14000u

 /*  ============================================================================
 *  @const  OFFSET_PSC0
 *
 *  @desc   Offset of the PSC0 from the base of the CFG
 *          memory.
 *  ============================================================================
 */
#define  OFFSET_PSC0                0x10000u

/** ============================================================================
 *  @name   DA8XXGEM_shmemInterface
 *
 *  @desc   Interface functions exported by the Shared Driver subcomponent.
 *  ============================================================================
 */
HAL_Interface DA8XXGEM_shmemInterface = {&DA8XXGEM_phyShmemInit,
                                          &DA8XXGEM_phyShmemExit,
                                          &DA8XXGEM_halBootCtrl,
                                          &DA8XXGEM_halIntCtrl,
                                          NULL,
                                          NULL,
                                          &DA8XXGEM_halPwrCtrl,
                                          NULL,
                                          NULL,
                                          NULL,
                                          NULL} ;


/* ============================================================================
 *  @func   DA8XXGEM_phyShmemInit
 *
 *  @desc   Initializes Shared/Davinci Gem device.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DA8XXGEM_phyShmemInit (IN Pvoid halObj)
{
    DSP_STATUS        status    = DSP_SOK ;
    DA8XXGEM_HalObj * halObject = NULL    ;
    MemMapInfo  mapInfo ;


    TRC_1ENTER ("DA8XXGEM_phySharedInit", halObj) ;

    DBC_Require (NULL != halObj) ;

    halObject = (DA8XXGEM_HalObj *) halObj ;

    /* map CHIP CFG regs */
    mapInfo.src  = CHIP_CFG_BASE  - (CHIP_CFG_BASE % PAGE_SIZE) ;
    mapInfo.size = CHIP_CFG_MAP_SIZE; // assuming page_size multiple
    mapInfo.memAttrs = MEM_UNCACHED ;
    status = MEM_Map (&mapInfo) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
        halObject->baseCfgBus = 0 ;
    }
    else {
        halObject->baseCfgBus = mapInfo.dst + (CHIP_CFG_BASE % PAGE_SIZE);
        halObject->offsetSysModule = OFFSET_SYSTEM_MODULE ;
        halObject->offsetPsc0      = OFFSET_PSC0 ;
    }


    TRC_1LEAVE ("DA8XXGEM_phySharedInit", status) ;

    return status ;
}


/* ============================================================================
 *  @func   DA8XXGEM_phyShmemExit
 *
 *  @desc   Finalizes Shared/Davinci Gem device.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DA8XXGEM_phyShmemExit (IN Pvoid halObj)
{
    DSP_STATUS     status = DSP_SOK ;
    MemUnmapInfo      unmapInfo                              ;
    DA8XXGEM_HalObj * halObject = NULL    ;

    TRC_1ENTER ("DA8XXGEM_phySharedExit", halObj) ;

    DBC_Require (NULL != halObj) ;

    halObject = (DA8XXGEM_HalObj *) halObj ;

    unmapInfo.addr = halObject->baseCfgBus - (halObject->baseCfgBus % PAGE_SIZE);
    unmapInfo.size = CHIP_CFG_MAP_SIZE;
    if (unmapInfo.addr != 0) {
        MEM_Unmap (&unmapInfo) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
        halObject->baseCfgBus = 0;
        halObject->offsetSysModule = 0;
        halObject->offsetPsc0      = 0;
    }


    (void) halObj ;

    TRC_1LEAVE ("DA8XXGEM_phySharedExit", status) ;

    return status ;
}


#if defined (__cplusplus)

}
#endif
