/** ============================================================================
 *  @file   dm6446gem_phy_shmem.c
 *
 *  @path   $(DSPLINK)/gpp/src/arch/DM6446GEM/shmem/WinCE/
 *
 *  @desc   WinCE shmem driver interface file.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */

 
 /*  ----------------------------------- OS Headers                  */
#include <windows.h>
#include <ceddk.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- Hardware Abstraction Layer  */
#include <dm6446gem_hal.h>
#include <dm6446gem_hal_boot.h>
#include <dm6446gem_hal_intgen.h>
#include <dm6446gem_hal_pwr.h>
#include <dm6446gem_phy_shmem.h>


#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_ARCH_SHMEM_PHY

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON TRC_SetReason (status,FID_C_ARCH_PHY_SHMEM,__LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/*  ============================================================================
 *  @const  OFFSET_SYSTEM_MODULE
 *
 *  @desc   Offset of the system module registers from the base of the CFG
 *          memory.
 *  ============================================================================
 */
#define  OFFSET_SYSTEM_MODULE   0x40000
#define DM6446GEM_BASE_CFG_BUS      0x01C00000
#define DM6446GEM_SYTEM_MODULE_SIZE     0x2000


/** ============================================================================
 *  @name   DM6446GEM_shmemInterface
 *
 *  @desc   Interface functions exported by the Shared Driver subcomponent.
 *  ============================================================================
 */
HAL_Interface DM6446GEM_shmemInterface = {&DM6446GEM_phyShmemInit,
                                          &DM6446GEM_phyShmemExit,
                                          &DM6446GEM_halBootCtrl,
                                          &DM6446GEM_halIntCtrl,
                                          NULL,
                                          NULL,
                                          &DM6446GEM_halPwrCtrl,
                                          NULL,
                                          NULL,
                                          NULL,
                                          NULL} ;


/* ============================================================================
 *  @func   DM6446GEM_phyShmemInit
 *
 *  @desc   Initializes Shared/Davinci Gem device.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DM6446GEM_phyShmemInit (IN Pvoid halObj)
{
    DSP_STATUS        status    = DSP_SOK ;
    DM6446GEM_HalObj * halObject = NULL    ;
    MemMapInfo  mapInfo ;
    UINT32  phys_system_module;
        

    TRC_1ENTER ("DM6446GEM_phySharedInit", halObj) ;

    DBC_Require (NULL != halObj) ;

    halObject = (DM6446GEM_HalObj *) halObj ;

    halObject->baseCfgBus      = 0x0  ;/* not used here in wince */
    /* since we map the pages in WINCE, lets restrict to what we need - SYSTEM MODULE and PSC */
    /* i.e. 0x01C40000 - 0x01C41FFF (8K) */
    phys_system_module = (DM6446GEM_BASE_CFG_BUS+OFFSET_SYSTEM_MODULE);
    mapInfo.src  = phys_system_module  - (phys_system_module % PAGE_SIZE) ;
    mapInfo.size = DM6446GEM_SYTEM_MODULE_SIZE ; // assuming page_size multiple 
    mapInfo.memAttrs = MEM_UNCACHED ;
    status = MEM_Map (&mapInfo) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
        halObject->offsetSysModule = 0 ;
    }
    else {
        halObject->offsetSysModule = mapInfo.dst + (phys_system_module % PAGE_SIZE);
    }
    
    TRC_1LEAVE ("DM6446GEM_phySharedInit", status) ;

    return status ;
}


/* ============================================================================
 *  @func   DM6446GEM_phyShmemExit
 *
 *  @desc   Finalizes Shared/Davinci Gem device.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DM6446GEM_phyShmemExit (IN Pvoid halObj)
{
    DSP_STATUS     status = DSP_SOK ;    
    MemUnmapInfo      unmapInfo                              ;
    DM6446GEM_HalObj * halObject = NULL    ;
    
    TRC_1ENTER ("DM6446GEM_phySharedExit", halObj) ;

    DBC_Require (NULL != halObj) ;
    
    halObject = (DM6446GEM_HalObj *) halObj ;
    
    unmapInfo.addr = halObject->offsetSysModule - (halObject->offsetSysModule % PAGE_SIZE);
    unmapInfo.size = DM6446GEM_SYTEM_MODULE_SIZE;
    if (unmapInfo.addr != 0) {
        MEM_Unmap (&unmapInfo) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
        halObject->offsetSysModule = 0 ;
    }

    (void) halObj ;

    TRC_1LEAVE ("DM6446GEM_phySharedExit", status) ;

    return status ;
}


#if defined (__cplusplus)

}
#endif
