/** ============================================================================
 *  @file   bin_loader.c
 *
 *  @path   $(DSPLINK)/gpp/src/gen/PrOS/
 *
 *  @desc   Implements the DSP Binary Loader.
 *          The binary loader loads a DSP executable, which is in the binary
 *          format. The loader expects a specific structure within the baseImage
 *          field of the loader object. This structure includes information such
 *          as the GPP address of the location of the binary file, the DSP load
 *          and run addresses, file size etc.
 *          Using this information, the loader directly writes the binary file
 *          contents in a single load into DSP memory at the specified DSP load
 *          address.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>
#include <loaderdefs.h>
#include <procdefs.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- Generic Functions           */
#include <gen_utils.h>
#include <bin_loader.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID         ID_GEN_BINLOADER

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define  SET_FAILURE_REASON   TRC_SetReason (status,                        \
                                             FID_C_GEN_BINLOADER,           \
                                             __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @const  MAX_SYMBOLS
 *
 *  @desc   Maximum number of symbols supported by the binary loader.
 *  ============================================================================
 */
#define  MAX_SYMBOLS         1u

/** ============================================================================
 *  @name   BINLOADER_SymbolName
 *
 *  @desc   Enumeration of symbol names supported by the binary loader.
 *
 *  @field  _DSPLINK_SHMBASEADDRESS
 *              DSPLINK shared memory base address.
 *  ============================================================================
 */
typedef enum {
    _DSPLINK_SHMBASEADDRESS = 0u
} BINLOADER_SymbolName ;

/*  ----------------------------------------------------------------------------
 *  @name   BINLOADER_Symbol
 *
 *  @desc   This structure defines a symbol object that stores information about
 *          a symbol supported by the binary loader.
 *
 *  @field  symName
 *              Name of the symbol.
 *  @field  symValue
 *              Value of the symbol.
 *  ----------------------------------------------------------------------------
 */
typedef struct BINLOADER_Symbol_tag {
    Char8           symName [DSP_MAX_STRLEN] ;
    Uint32          symValue ;
} BINLOADER_Symbol ;


/** ============================================================================
 *  @name   BINLOADER_Interface
 *
 *  @desc   Interface table for Binary loader.
 *  ============================================================================
 */
LOADER_Interface BINLOADER_Interface = {
    &BINLOADER_init,
    &BINLOADER_exit,
    &BINLOADER_load,
    &BINLOADER_loadSection,
    &BINLOADER_getSymbolAddress
} ;

/*  ----------------------------------------------------------------------------
 *  @name   BINLOADER_symArray
 *
 *  @desc   Array of symbol name and value pairs supported by the binary loader.
 *  ----------------------------------------------------------------------------
 */
STATIC BINLOADER_Symbol BINLOADER_symArray [MAX_SYMBOLS] ;


/** ============================================================================
 *  @func   BINLOADER_init
 *
 *  @desc   Initializes the context object for loading a base image file or
 *          a section. This function is required to be called before any other
 *          function is called from this sub component.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
BINLOADER_init (IN  ProcessorId      procId,
                IN  Pstr             baseImage,
                IN  LoaderInitArgs * args,
                OUT Pvoid *          objCtx)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_4ENTER ("BINLOADER_init", procId, baseImage, args, objCtx) ;

    DBC_Require (IS_VALID_PROCID (procId)) ;
    DBC_Require (baseImage != NULL) ;
    DBC_Require (args      != NULL) ;
    DBC_Require (objCtx    != NULL) ;

    *objCtx = (Pvoid) baseImage ;

    TRC_1LEAVE ("BINLOADER_init", status) ;

    return status ;
}


/** ============================================================================
 *  @func   BINLOADER_exit
 *
 *  @desc   Deallocates the object(s) created by BINLOADER_init function
 *          and releases the context.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
BINLOADER_exit (IN  Pvoid objCtx)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("BINLOADER_exit", objCtx) ;

    DBC_Require (objCtx != NULL) ;

    /* Nothing to be done to finalize the Binary Loader. */

    TRC_1LEAVE ("BINLOADER_exit", status) ;

    return status ;
}


/** ============================================================================
 *  @func   BINLOADER_load
 *
 *  @desc   Loads the Binary format file on the DSP. This is called by
 *          PMGR_PROC_load through the function pointer table.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
BINLOADER_load (IN  ProcessorId     procId,
                IN  LoaderObject *  loaderObj,
                IN  Uint32          argc,
                IN  Char8 **        argv,
                OUT Uint32 *        entryPt)
{
    DSP_STATUS            status = DSP_SOK ;
    BINLOADER_ImageInfo * imageInfo ;

    TRC_5ENTER ("BINLOADER_load", procId, loaderObj, argc, argv, entryPt) ;

    DBC_Require (IS_VALID_PROCID (procId)) ;
    DBC_Require (loaderObj != NULL) ;
    DBC_Require (entryPt != NULL) ;
    DBC_Require (   ((argc == 0) && (argv == NULL))
                 || ((argc != 0) && (argv != NULL))) ;

    DBC_Assert (loaderObj->objCtx != NULL) ;

    imageInfo = (BINLOADER_ImageInfo *) loaderObj->objCtx ;

    status = GEN_Strcpyn (BINLOADER_symArray [_DSPLINK_SHMBASEADDRESS].symName,
                          DRV_SHMBASESYMBOL,
                          DSP_MAX_STRLEN) ;
    /* Parameters are non-NULL so success is assured. */
    DBC_Assert (DSP_SUCCEEDED (status)) ;

    BINLOADER_symArray [_DSPLINK_SHMBASEADDRESS].symValue =
                                                       imageInfo->shmBaseAddr ;

    /* Load the binary file at the specified DSP load address. */
    status = (*(loaderObj->fnWriteDspMem)) (procId,
                                            (imageInfo->dspLoadAddr),
                                            Endianism_Default,
                                            imageInfo->fileSize,
                                            (Uint8 *) (imageInfo->gppAddr)) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }
    else {
        /* Return the address from which DSP should be run. */
        *entryPt = imageInfo->dspRunAddr ;
    }

    TRC_1LEAVE ("BINLOADER_load", status) ;

    return status ;
}


/** ============================================================================
 *  @func   BINLOADER_loadSection
 *
 *  @desc   Loads a section from the DSP executable onto the DSP. This is called
 *          by PMGR_PROC_loadSection through the function pointer table.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
BINLOADER_loadSection (IN  ProcessorId     procId,
                       IN  LoaderObject *  loaderObj,
                       IN  Uint32          sectId)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_3ENTER ("BINLOADER_loadSection", procId, loaderObj, sectId) ;

    status = DSP_ENOTIMPL ;
    SET_FAILURE_REASON ;

    TRC_1LEAVE ("BINLOADER_loadSection", status) ;

    return status ;
}


/** ============================================================================
 *  @func   BINLOADER_getSymbolAddress
 *
 *  @desc   Gets the DSP address corresponding to a symbol within a DSP
 *          executable currently loaded on the DSP.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
BINLOADER_getSymbolAddress (IN   ProcessorId     procId,
                            IN   Void *          objCtx,
                            IN   Char8 *         symbolName,
                            OUT  Uint32 *        dspAddr)
{
    DSP_STATUS status     = DSP_SOK ;
    Int32      cmpResult  = -1l ;
    Uint32     i ;

    TRC_4ENTER ("BINLOADER_getSymbolAddress",
                procId,
                objCtx,
                symbolName,
                dspAddr) ;

    DBC_Require (IS_VALID_PROCID (procId)) ;
    DBC_Require (objCtx     != NULL) ;
    DBC_Require (symbolName != NULL) ;
    DBC_Require (dspAddr    != NULL) ;

    if (    (!(IS_VALID_PROCID (procId)))
        ||  (objCtx     == NULL)
        ||  (symbolName == NULL)
        ||  (dspAddr    == NULL)) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        *dspAddr = (Uint32) NULL ;
        for (i = 0 ; (i < MAX_SYMBOLS) && (DSP_SUCCEEDED (status)) ; i++) {
            status = GEN_Strcmp (BINLOADER_symArray [i].symName,
                                 symbolName,
                                 &cmpResult) ;
            if (DSP_SUCCEEDED (status)) {
                if (cmpResult == 0) {
                    *dspAddr = BINLOADER_symArray [i].symValue ;
                }
            }

        }

        if ((*dspAddr) == ((Uint32) NULL)) {
            /* Symbol lookup not supported for any other symbols. */
            status = DSP_ENOTSUPPORTED ;
        }
    }

    TRC_1LEAVE ("BINLOADER_getSymbolAddress", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
