/** ============================================================================
 *  @file   static_loader.c
 *
 *  @path   $(DSPLINK)/gpp/src/gen/PrOS/
 *
 *  @desc   Implements the DSP Static Loader.
 *          This loader is used when the DSP executable is statically linked
 *          into the ARM-side executable. This is possible when the ARM and DSP
 *          memory space is unified, as in the Jacinto platform.
 *          In this loader, no separate load of the DSP executable is required,
 *          since it gets loaded automatically when the ARM executable is
 *          loaded.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>
#include <_loaderdefs.h>
#include <procdefs.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- Generic Functions           */

#include <static_loader.h>
#include <coff_64x.h>
#include <gen_utils.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID         ID_GEN_STATICLOADER

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define  SET_FAILURE_REASON   TRC_SetReason (status,                        \
                                             FID_C_GEN_STATICLOADER,           \
                                             __LINE__)
#else
#define SET_FAILURE_REASON  { }
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @const  MAX_SYMBOLS
 *
 *  @desc   Maximum number of symbols supported by the static loader.
 *  ============================================================================
 */
#define  MAX_SYMBOLS         1u

/** ============================================================================
 *  @name   STATICLOADER_SymbolName
 *
 *  @desc   Enumeration of symbol names supported by the static loader.
 *
 *  @field  _DSPLINK_SHMBASEADDRESS
 *              DSPLINK shared memory base address.
 *  ============================================================================
 */
typedef enum {
    _DSPLINK_SHMBASEADDRESS = 0
} STATICLOADER_SymbolName ;

/*  ----------------------------------------------------------------------------
 *  @name   STATICLOADER_Symbol
 *
 *  @desc   This structure defines a symbol object that stores information about
 *          a symbol supported by the static loader.
 *
 *  @field  symName
 *              Name of the symbol.
 *  @field  symValue
 *              Value of the symbol.
 *  ----------------------------------------------------------------------------
 */
typedef struct STATICLOADER_Symbol_tag {
    Char8           symName [DSP_MAX_STRLEN] ;
    Uint32          symValue ;
} STATICLOADER_Symbol ;

/** ============================================================================
 *  @name   LDRV_LinkCfgPtr
 *
 *  @desc   Global pointer containing DSPLINK configuration information for the
 *          driver.
 *  ============================================================================
 */
extern LINKCFG_Object * LDRV_LinkCfgPtr ;


/** ============================================================================
 *  @name   STATICLOADER_Interface
 *
 *  @desc   Interface table for Static loader.
 *  ============================================================================
 */
LOADER_Interface STATICLOADER_Interface = {
    &STATICLOADER_init,
    &STATICLOADER_exit,
    &STATICLOADER_load,
    &STATICLOADER_loadSection,
    &STATICLOADER_getSymbolAddress
} ;

/*  ----------------------------------------------------------------------------
 *  @name   STATICLOADER_symArray
 *
 *  @desc   Array of symbol name and value pairs supported by the static loader.
 *  ----------------------------------------------------------------------------
 */
STATIC STATICLOADER_Symbol STATICLOADER_symArray [MAX_SYMBOLS] ;


/** ============================================================================
 *  @func   STATICLOADER_init
 *
 *  @desc   Initializes the context object for loading a base image file or
 *          a section. This function is required to be called before any other
 *          function is called from this sub component.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
STATICLOADER_init (IN  ProcessorId      procId,
                   IN  Pstr             baseImage,
                   IN  LoaderInitArgs * args,
                   OUT Pvoid *          objCtx)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_4ENTER ("STATICLOADER_init", procId, baseImage, args, objCtx) ;

    DBC_Require (IS_VALID_PROCID (procId)) ;
    DBC_Require (baseImage != NULL) ;
    DBC_Require (args      != NULL) ;
    DBC_Require (objCtx    != NULL) ;

    *objCtx = (Pvoid) baseImage ;

    TRC_1LEAVE ("STATICLOADER_init", status) ;

    return status ;
}


/** ============================================================================
 *  @func   STATICLOADER_exit
 *
 *  @desc   Deallocates the object(s) created by STATICLOADER_init function
 *          and releases the context.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
STATICLOADER_exit (IN  Pvoid objCtx)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("STATICLOADER_exit", objCtx) ;

    /* Nothing to be done to finalize the Static Loader. */

    TRC_1LEAVE ("STATICLOADER_exit", status) ;

    return status ;
}


/** ============================================================================
 *  @func   STATICLOADER_load
 *
 *  @desc   Loads the DSP executable that has been statically linked into the
 *          ARM executable. This is called by PMGR_PROC_load through the
 *          function pointer table.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
STATICLOADER_load (IN  ProcessorId     procId,
                   IN  LoaderObject *  loaderObj,
                   IN  Uint32          argc,
                   IN  Char8 **        argv,
                   OUT Uint32 *        entryPt)
{
    DSP_STATUS               status  = DSP_SOK ;
    DspArch                  dspArch = DspArch_C64x_Bios5 ;
    STATICLOADER_ImageInfo * imageInfo ;
    LINKCFG_Dsp            * dspObj ;

    TRC_5ENTER ("STATICLOADER_load",
                procId,
                loaderObj,
                argc,
                argv,
                entryPt) ;

    DBC_Require (IS_VALID_PROCID (procId)) ;
    DBC_Require (loaderObj != NULL) ;
    DBC_Require (entryPt != NULL) ;
    DBC_Require (   ((argc == 0) && (argv == NULL))
                 || ((argc != 0) && (argv != NULL))) ;

    imageInfo = (STATICLOADER_ImageInfo *) loaderObj->objCtx ;
    DBC_Assert (imageInfo != NULL) ;

    status = GEN_Strcpyn (
                      STATICLOADER_symArray [_DSPLINK_SHMBASEADDRESS].symName,
                      DRV_SHMBASESYMBOL,
                      DSP_MAX_STRLEN) ;
    /* Parameters are non-NULL so success is assured. */
    DBC_Assert (DSP_SUCCEEDED (status)) ;

    STATICLOADER_symArray [_DSPLINK_SHMBASEADDRESS].symValue =
                                                      imageInfo->shmBaseAddr ;

    dspObj   = LDRV_LinkCfgPtr->dspConfigs [procId]->dspObject ;

    /* Write the user-specified arguments into the .args section */
    if (argc > 0) {
        if (argv != NULL) {

            /* Default indicates DSP/BIOS version is DspArch_C64x_Bios5 */
            if ( (DspArch) dspObj->dspArch == (DspArch) DspArch_C64x) {
                dspArch = DspArch_C64x_Bios5 ;
            }

            status = COFF_fillArgsBuffer_64x ((DspArch) dspArch,
                                              argc,
                                              argv,
                                              imageInfo->argsSize,
                                              imageInfo->argsAddr,
                                              loaderObj->maduSize,
                                              loaderObj->endian,
                                              (Void *) imageInfo->argsAddr) ;

            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
        }
        else {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }
    }

    /* Return the address from which DSP should be run. For the static loader
     * this comes from the generated files
     */
    if (DSP_SUCCEEDED (status)) {
        *entryPt = imageInfo->dspRunAddr ;
    }

    TRC_1LEAVE ("STATICLOADER_load", status) ;

    return status ;
}


/** ============================================================================
 *  @func   STATICLOADER_loadSection
 *
 *  @desc   Loads a section from the DSP executable onto the DSP. This is called
 *          by PMGR_PROC_loadSection through the function pointer table.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
STATICLOADER_loadSection (IN  ProcessorId     procId,
                          IN  LoaderObject *  loaderObj,
                          IN  Uint32          sectId)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_3ENTER ("STATICLOADER_loadSection", procId, loaderObj, sectId) ;

    status = DSP_ENOTIMPL ;
    SET_FAILURE_REASON ;

    TRC_1LEAVE ("STATICLOADER_loadSection", status) ;

    return status ;
}


/** ============================================================================
 *  @func   STATICLOADER_getSymbolAddress
 *
 *  @desc   Gets the DSP address corresponding to a symbol within a DSP
 *          executable currently loaded on the DSP.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
STATICLOADER_getSymbolAddress (IN   ProcessorId     procId,
                               IN   Void *          objCtx,
                               IN   Char8 *         symbolName,
                               OUT  Uint32 *        dspAddr)
{
    DSP_STATUS status     = DSP_SOK ;
    Int32      cmpResult  = -1 ;
    Uint32     i ;

    TRC_4ENTER ("STATICLOADER_getSymbolAddress",
                procId,
                objCtx,
                symbolName,
                dspAddr) ;

    DBC_Require (IS_VALID_PROCID (procId)) ;
    DBC_Require (objCtx     != NULL) ;
    DBC_Require (symbolName != NULL) ;
    DBC_Require (dspAddr    != NULL) ;

    if (    (!(IS_VALID_PROCID (procId)))
        ||  (objCtx     == NULL)
        ||  (symbolName == NULL)
        ||  (dspAddr    == NULL)) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        *dspAddr = (Uint32) NULL ;
        for (i = 0 ; (i < MAX_SYMBOLS) && (DSP_SUCCEEDED (status)) ; i++) {
            status = GEN_Strcmp (STATICLOADER_symArray [i].symName,
                                 symbolName,
                                 &cmpResult) ;
            if (DSP_SUCCEEDED (status)) {
                if (cmpResult == 0) {
                    *dspAddr = STATICLOADER_symArray [i].symValue ;
                }
            }

        }

        if ((*dspAddr) == ((Uint32) NULL)) {
            /* Symbol lookup not supported for any other symbols. */
            status = DSP_ENOTSUPPORTED ;
        }
    }

    TRC_1LEAVE ("STATICLOADER_getSymbolAddress", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
