/** ============================================================================
 *  @file   prcs.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/PrOS/
 *
 *  @desc   Implementation of sub-component PRCS.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <error.h>
#include <types.h>
#include <kernel.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component ID of this module.
 *  ============================================================================
 */
#define  COMPONENT_ID        ID_OSAL_PRCS

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  TRC_SetReason (status, FID_C_OSAL_PRCS, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG)  */

/** ============================================================================
 *  @const  MAX_NAME_LEN
 *
 *  @desc   Maximum length of task name.
 *  ============================================================================
 */
#define MAX_NAME_LEN         8u


/** ============================================================================
 *  @name   PrcsObject
 *
 *  @desc   Structure to store information regarding current process/thread.
 *
 *  @field  signature
 *              Signature of this structure.
 *  @field  handleToProcess
 *              Handle to current process.
 *  @field  handleToThread
 *              Handle to current thread.
 *  @field  priorityOfProcess
 *              Priority of current process.
 *  @field  priorityOfThread
 *              Priority of current Thread.
 *  ============================================================================
 */
struct PrcsObject_tag {
    Uint32 signature         ;
    Void * handleToProcess   ;
    Void * handleToThread    ;
    Int32  priorityOfProcess ;
    Int32  priorityOfThread  ;
} ;


/** ============================================================================
 *  @name   gruntcb
 *
 *  @desc   pointer to current running task.
 *  ============================================================================
 */
extern volatile struct tcb * gruntcb ;

/** ============================================================================
 *  @name   PRCS_IsInitialized
 *
 *  @desc   Flag to keep track of initialization of this subcomponent.
 *  ============================================================================
 */
STATIC Bool PRCS_IsInitialized = FALSE ;


/** ============================================================================
 *  @func   PRCS_Initialize
 *
 *  @desc   Initializes the PRCS subcomponent.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_Initialize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("PRCS_Initialize") ;

    DBC_Require (PRCS_IsInitialized == FALSE) ;

    if (PRCS_IsInitialized == FALSE) {
        PRCS_IsInitialized = TRUE ;
    }

    TRC_1LEAVE ("PRCS_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_Finalize
 *
 *  @desc   Finalizes the PRCS component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_Finalize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("PRCS_Finalize") ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;

    if (PRCS_IsInitialized == TRUE) {
        PRCS_IsInitialized = FALSE ;
    }

    TRC_1LEAVE ("PRCS_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_Create
 *
 *  @desc   Creates a PrcsObject and populates it with information to identify
 *          the client.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_Create (OUT PrcsObject ** prcsObj, IN     Void * optArgs)
{
    DSP_STATUS  status   = DSP_SOK  ;
    ER          erStatus = E_OK     ;
    ID        p_tskId  ;
    PRI       p_tskpri ;

    TRC_1ENTER ("PRCS_Create", prcsObj) ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;
    DBC_Require (prcsObj != NULL) ;

    (Void) optArgs ;

    if (prcsObj == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        *prcsObj = NULL ;

        status = MEM_Alloc ((Pvoid *) prcsObj,
                            sizeof (PrcsObject),
                            MEM_DEFAULT) ;
    }

    if (DSP_SUCCEEDED (status)) {
        SYNC_ProtectionStart () ;
        /** --------------------------------------------------------------------
         *  Reference the task and get the task information.
         *  --------------------------------------------------------------------
         */
        erStatus = get_tid (&p_tskId) ;
        if (erStatus != E_OK) {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
        else {
            erStatus = get_pri (p_tskId, &p_tskpri) ;
            if (erStatus != E_OK) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            else {
                /** --------------------------------------------------------------------
                 *  Populate the process object with the task information only.
                 *  Since in PrOS we have only one process.
                 *  --------------------------------------------------------------------
                 */
                if (DSP_SUCCEEDED (status)) {
                    (*prcsObj)->signature         = SIGN_PRCS                     ;
                    (*prcsObj)->handleToThread    = (Void *) p_tskId ;
                    (*prcsObj)->handleToProcess   = NULL                          ;
                    (*prcsObj)->priorityOfThread  = p_tskpri ;
                    (*prcsObj)->priorityOfProcess = 0                             ;
                }
            }
        }
        SYNC_ProtectionEnd () ;
    }

    DBC_Ensure (   (   DSP_SUCCEEDED (status)
                    && (prcsObj != NULL)
                    && IS_OBJECT_VALID(*prcsObj, SIGN_PRCS))
                || DSP_FAILED (status)) ;

    TRC_1LEAVE ("PRCS_Create", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_Delete
 *
 *  @desc   Frees up resources used by the specified object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_Delete (IN PrcsObject * prcsObj)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_1ENTER ("PRCS_Delete", prcsObj) ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;
    DBC_Require (prcsObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (prcsObj, SIGN_PRCS)) ;

    if (IS_OBJECT_VALID (prcsObj, SIGN_PRCS)) {
        prcsObj->signature = SIGN_NULL ;
        status = FREE_PTR (prcsObj) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
    }
    else {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("PRCS_Delete", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_IsEqual
 *
 *  @desc   Compares two clients to check if they are "equal".
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_IsEqual (IN  PrcsObject *  client1,
              IN  PrcsObject *  client2,
              OUT Bool *        isEqual)
{
    DSP_STATUS  status = DSP_SOK ;
    TRC_3ENTER ("PRCS_IsEqual", client1, client2, isEqual) ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;
    DBC_Require (IS_OBJECT_VALID (client1, SIGN_PRCS)) ;
    DBC_Require (IS_OBJECT_VALID (client2, SIGN_PRCS)) ;
    DBC_Require (isEqual != NULL) ;

    if (   (IS_OBJECT_VALID (client1, SIGN_PRCS) == FALSE)
        || (IS_OBJECT_VALID (client2, SIGN_PRCS) == FALSE)
        || (isEqual == NULL)) {

        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        /** --------------------------------------------------------------------
         *  In PrOS we have only one process, so we compare only task id.
         *  --------------------------------------------------------------------
         */
        if ((client1->handleToThread  == client2->handleToThread)) {
            *isEqual = TRUE ;
        }
        else {
            *isEqual = FALSE ;
        }
    }

    TRC_1LEAVE ("PRCS_IsEqual", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_IsSameContext
 *
 *  @desc   Checks if the two clients share same context (address space).
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_IsSameContext (IN  PrcsObject *  client1,
                    IN  PrcsObject *  client2,
                    OUT Bool *        isSame)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_3ENTER ("PRCS_IsSameContext", client1, client2, isSame) ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;
    DBC_Require (IS_OBJECT_VALID (client1, SIGN_PRCS)) ;
    DBC_Require (IS_OBJECT_VALID (client2, SIGN_PRCS)) ;
    DBC_Require (isSame != NULL) ;

    /** ------------------------------------------------------------------------
     *  In PrOS we have only one process. So, return TRUE always.
     *  ------------------------------------------------------------------------
     */
    *isSame = TRUE ;

    TRC_1LEAVE ("PRCS_IsSameContext", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
