/** ============================================================================
 *  @file   dpc.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/WinCE/
 *
 *  @desc   Implementation of sub-component DPC.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <windows.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- Generic Functions           */
#include <gen_utils.h>

#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and sub-component Identifier.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_OSAL_DPC

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  TRC_SetReason (status, FID_C_OSAL_DPC, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */

/** ============================================================================
 *  @const  MAX_DPC
 *
 *  @desc   Maximum number of DPCs supported.
 *  ============================================================================
 */
#define MAX_DPC             32

/** ============================================================================
 *  @const  INVALID_INDEX
 *
 *  @desc   Invalid index of a DPC object.
 *  ============================================================================
 */
#define INVALID_INDEX       0xFFFFFFFF

/** ============================================================================
 *  @const  DPC_THREAD_PRIORITY
 *
 *  @desc   Thread priority for the DPC threads
 *  ============================================================================
 */

#define DPC_THREAD_PRIORITY 125



/** ============================================================================
 *  @name   DpcObject_tag
 *
 *  @desc   This structure defines the association between DpcObjects and their
 *          corresponding tasklets. It also contains a bitmap for tracking used
 *          Dpc objects.
 *
 *  @field  isInit
 *              Initialize flag.
 *  @field  proc
 *              Registered function (DPC entry point).
 *  @field  param
 *              Argument to be passed to DPC entry function.
 *  @field  count
 *              Activation counter.
 *  @field  exit
 *              Indicating it is safe to exit.
 *  @field  hDpcEvent
 *              Event that is used for signalling DPC threads
 *  @field  hDpcThread
 *              Kernel thread.
 *
 *  @see    None
 *  ============================================================================
 */
struct DpcObject_tag {
    Bool                 isInit ;
    FnDpcProc            proc   ;
    Pvoid                param  ;
    volatile long        count  ;
    volatile long        exit   ;    
    HANDLE               hDpcEvent   ;
    HANDLE               hDpcThread   ;
} ;


/** ============================================================================
 *  @name   DpcArray
 *
 *  @desc   Subcomponent variable to keep of all the DPCs and associated
 *          hisrs created for 'Link.
 *  ============================================================================
 */
DpcObject DpcArray [MAX_DPC] ;

/** ============================================================================
 *  @name   DPC_IsInitialized
 *
 *  @desc   Flag to keep track of initialization of this subcomponent.
 *  ============================================================================
 */
STATIC Bool DPC_IsInitialized = FALSE ;


/** ============================================================================
 *  @name   DpcLockObj
 *
 *  @desc   Lock for using the DPC resources.
 *  ============================================================================
 */
STATIC Pvoid DpcLockObj = NULL ;


/** ----------------------------------------------------------------------------
 *  @func   DPC_Callback
 *
 *  @desc   This function gets invoked by the OS when it schedules a tasklet
 *          to execute.
 *
 *  @arg    index
 *              Indicates the index in DPC object table.
 *
 *  @ret    None
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    DPC_Schedule
 *  ----------------------------------------------------------------------------
 */
STATIC
int
DPC_Callback (LPVOID arg) ;


/** ============================================================================
 *  @func   DPC_Initialize
 *
 *  @desc   This function initializes the DPC module. It initializes the
 *          global area (DPC_DpcWorkQMap structure) for holding all the
 *          DPC objects and marks the UsedDPCs bitmap to indicate that no
 *          DPCs are currently in use.
 *
 *  @modif  DPC_DpcWorkQMap
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Initialize ()
{
    DSP_STATUS  status = DSP_SOK ;
    Uint32      i                ;

    TRC_0ENTER ("DPC_Initialize") ;

    if (DPC_IsInitialized == FALSE) {

        /* Create the DPC global lock */
        DpcLockObj = SYNC_SpinLockCreate () ;

        for (i = 0u ; i < MAX_DPC ; i++) {
            memset (&DpcArray [i], 0, sizeof (DpcObject)) ;
            DpcArray [i].isInit = FALSE ;
        }

        DPC_IsInitialized = TRUE ;
    }

    DBC_Ensure (DPC_IsInitialized == TRUE) ;

    TRC_1LEAVE ("DPC_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Finalize
 *
 *  @desc   This function provides an interface to exit from the DPC module.
 *          It frees up all the used DPCs and releases all the resources used by
 *          this module.
 *
 *  @modif  DPC_DpcWorkQMap
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Finalize ()
{
    DSP_STATUS  status = DSP_SOK ;
    Uint32      i      = 0       ;

    TRC_0ENTER ("DPC_Finalize") ;

    DBC_Require (DPC_IsInitialized == TRUE) ;

    if (DPC_IsInitialized == TRUE) {
        SYNC_SpinLockStartEx (DpcLockObj) ;
        for (i = 0u ; i < MAX_DPC ; i++) {
            if (DpcArray [i].isInit == TRUE) {
                /* Finalize if any active DPC found */
                DPC_Delete (&DpcArray [i]) ;
            }
        }
        DPC_IsInitialized = FALSE ;
        SYNC_SpinLockEndEx (DpcLockObj, 0u) ;
        /* Now it is safe to delete the lock */
        SYNC_SpinLockDelete (DpcLockObj) ;
        DpcLockObj = NULL ;
    }

    DBC_Ensure (DPC_IsInitialized == FALSE) ;

    TRC_1LEAVE ("DPC_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Create
 *
 *  @desc   Creates a DPC object and returns it after populating
 *          relevant fields. It sets a bit in UsedDPCs bitmask.
 *
 *  @modif  DPC_DpcWorkQMap
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Create (IN  FnDpcProc    userDPCFn,
            IN  Pvoid        dpcArgs,
            OUT DpcObject ** dpcObj)
{
    DSP_STATUS  status = DSP_SOK ;

    DSP_STATUS  tmpstatus = DSP_SOK ;
    Uint32      index  = 0       ;

    Uint32      threadPRI = 0;
    Uint32      bfrSz=0;

    TRC_3ENTER ("DPC_Create", userDPCFn, dpcArgs, dpcObj) ;

    DBC_Require (DPC_IsInitialized == TRUE) ;
    DBC_Require (userDPCFn != NULL) ;
    DBC_Require (dpcObj != NULL) ;

    if ((userDPCFn == NULL) || (dpcObj == NULL)) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        /* Find a free index */
        SYNC_SpinLockStartEx (DpcLockObj) ;
        for (index = 0 ; index < MAX_DPC ; index++) {
            if (DpcArray [index].isInit == FALSE) {
                break ;
            }
        }

        if (index == MAX_DPC) {
            status = DSP_ERESOURCE ;
            SET_FAILURE_REASON ;
        }
        else
        {   
            DpcArray [index].hDpcEvent = CreateEvent( NULL, FALSE, FALSE, NULL);
            if (DpcArray [index].hDpcEvent == NULL) 
            {
                /* Creation of Event Object failed. Cleanup */
                status = DSP_ERESOURCE ;
                *dpcObj = NULL ;
                SET_FAILURE_REASON ;
            }
            else
            {
                /* Create the kernel thread */
                DpcArray [index].hDpcThread = CreateThread(NULL,0,
                                                            DPC_Callback,
                                                            (LPVOID) &(DpcArray [index]), 
                                                            CREATE_SUSPENDED,
                                                            NULL) ;

                if (DpcArray [index].hDpcThread != NULL) 
                {
                    DpcArray [index].proc  = userDPCFn ;
                    DpcArray [index].param = dpcArgs ;

                    /* Initialize the state, count and exit flags */
                    InterlockedExchange(&DpcArray [index].count, 0) ;                    
                    InterlockedExchange(&DpcArray[index].exit, 0);
                    
                    
                    DpcArray [index].isInit = TRUE ;
                    *dpcObj = &DpcArray[index] ;
                    


                    bfrSz=sizeof(threadPRI);
                    tmpstatus=Reg_GetKeyValue(HKEY_LOCAL_MACHINE,"Drivers\\BuiltIn\\DSPLINK","DPC_THPRI",(Uint8 *)&threadPRI,&bfrSz);
                    if (!DSP_SUCCEEDED(tmpstatus))
                    {
                        threadPRI=DPC_THREAD_PRIORITY;
                    }
                    CeSetThreadPriority(DpcArray[index].hDpcThread, threadPRI);

                    /* Release the global lock */
                    SYNC_SpinLockEndEx (DpcLockObj, 0u) ;                    
                    
                    
                    /* Finally start the kernel thread */
                    ResumeThread(DpcArray [index].hDpcThread) ;

                    /* Again take the global lock */
                    SYNC_SpinLockStartEx (DpcLockObj) ;
                }
                else
                {
                    *dpcObj = NULL ;
                    status = DSP_EFAIL ;
                    SET_FAILURE_REASON ;
                }
            }
        }
        SYNC_SpinLockEndEx (DpcLockObj, 0u) ;
    }

    TRC_1LEAVE ("DPC_Create", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Delete
 *
 *  @desc   Destroys the DPC object by resetting the UsedDPCs bitmask.
 *
 *  @modif  DPC_DpcWorkQMap
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Delete (IN DpcObject * dpcObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("DPC_Delete", dpcObj) ;

    DBC_Require (DPC_IsInitialized == TRUE) ;
    DBC_Require (dpcObj != NULL) ;

    if (dpcObj->isInit == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        InterlockedExchange (&dpcObj->count, 0) ;
        /* Set the exit flag */
        InterlockedExchange(&dpcObj->exit,1);
        /* uninitialize the flag */
        dpcObj->isInit = FALSE ;
        /* wait for the thread to stop */
        SetEvent(dpcObj->hDpcEvent);
        WaitForSingleObject(dpcObj->hDpcThread, INFINITE);

        /* close the handle */
        CloseHandle(dpcObj->hDpcThread);
        dpcObj->hDpcThread = NULL;
        if (!CloseHandle (dpcObj->hDpcEvent)) {
            status = DSP_EINVALIDARG;
            SET_FAILURE_REASON;
        }
        /* Clear out the object */
        memset ((Uint8 *) dpcObj, 0, sizeof (DpcObject)) ;
    }

    TRC_1LEAVE ("DPC_Delete", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Cancel
 *
 *  @desc   Cancels the DPC.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Cancel (IN DpcObject * dpcObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("DPC_Cancel", dpcObj) ;

    DBC_Require (DPC_IsInitialized == TRUE) ;
    DBC_Require (dpcObj != NULL) ;

    if (dpcObj->isInit == FALSE) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        /* Clear the count */
        InterlockedExchange (&dpcObj->count, 0) ;
    }

    TRC_1LEAVE ("DPC_Cancel", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DPC_Schedule
 *
 *  @desc   This function schedules the DPC object, causing the user defined
 *          function associated with it to be invoked at a later point of time.
 *          This function is called from ISR context.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
DPC_Schedule (IN  DpcObject * dpcObj)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("DPC_Schedule", dpcObj) ;

    DBC_Require (DPC_IsInitialized == TRUE) ;
    DBC_Require (dpcObj != NULL) ;

    if (dpcObj->isInit == FALSE) {
       status = DSP_EPOINTER ;
       SET_FAILURE_REASON ;
    }
    else {
        /* If DPC is not exited only then wake up */
        if (InterlockedExchangeAdd (&dpcObj->exit,0) == 0) {
            InterlockedIncrement (&dpcObj->count);
            SetEvent(dpcObj->hDpcEvent);
        }
    }

    TRC_0LEAVE ("DPC_Schedule") ;

    return DSP_SOK ;
}


/** ============================================================================
 *  @func   DPC_Disable
 *
 *  @desc   Disables execution of DPCs.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
DPC_Disable ()
{
    Int16 i ;

    TRC_0ENTER ("DPC_Disable") ;

    for (i = 0 ; i < MAX_DPC ; i++) {
        if (DpcArray [i].isInit == TRUE) {
            /* Take the enable mutex, so that DPC waits for it */
            SuspendThread (DpcArray[i].hDpcThread);
        }
    }

    TRC_0LEAVE ("DPC_Disable") ;
}


/** ============================================================================
 *  @func   DPC_Enable
 *
 *  @desc   Enables execution of DPCs.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
DPC_Enable ()
{
    Int16 i ;

    TRC_0ENTER ("DPC_Enable") ;

    for (i = 0 ; i < MAX_DPC ; i++) {
        if (DpcArray [i].isInit == TRUE) {
            /* release the lock, so that DPC takes it and run */
            ResumeThread (DpcArray[i].hDpcThread);
        }
    }

    TRC_0LEAVE ("DPC_Enable") ;
}


#if defined (DDSP_DEBUG)
/** ============================================================================
 *  @func   DPC_Debug
 *
 *  @desc   This function prints the current status of DPC objects in
 *          the 'Link.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
DPC_Debug ()
{
    Uint32 i = 0 ;

    TRC_0ENTER ("DPC_Debug") ;

    for (i = 0 ; i < MAX_DPC ; i++) {
        TRC_2PRINT (TRC_LEVEL1,
                    "DpcArray [%d].isInit : [0x%x]\n",
                    i,
                    DpcArray [i].isInit) ;
        TRC_2PRINT (TRC_LEVEL1,
                    "DpcArray [%d].proc  : [0x%x]\n",
                    i,
                    DpcArray [i].proc) ;
        TRC_2PRINT (TRC_LEVEL1,
                    "DpcArray [%d].param : [0x%x]\n",
                    i,
                    DpcArray [i].param) ;

        TRC_2PRINT (TRC_LEVEL1,
                    "DpcArray [%d].exit : [0x%x]\n",
                    i,
                    InterlockedExchangeAdd( &(DpcArray [i].exit),0)) ;
        TRC_2PRINT (TRC_LEVEL1,
                    "DpcArray [%d].count : [0x%x]\n",
                    i,
                    InterlockedExchangeAdd(&(DpcArray [i].count),0)) ;

        TRC_2PRINT (TRC_LEVEL1,
                    "DpcArray [%d].thread : [0x%x]\n",
                    i,
                    DpcArray [i].hDpcThread) ;
        TRC_2PRINT (TRC_LEVEL1,
                    "DpcArray [%d].event : [0x%x]\n",
                    i,
                    DpcArray [i].hDpcEvent) ;
    }

    TRC_0LEAVE ("DPC_Debug") ;
}
#endif /* defined (DDSP_DEBUG) */

/*  ----------------------------------------------------------------------------
 *  @func   DPC_Callback
 *
 *  @desc   This function gets invoked when a tasklet is scheduled to run
 *          by kernel. It services all the pending DPC requests.
 *
 *  @modif  None
 *  ----------------------------------------------------------------------------
 */
STATIC
int
DPC_Callback (LPVOID arg)
{
    DpcObject * dpcObj      = (DpcObject *)arg;
    DSP_STATUS  status      = DSP_SOK ;
    DWORD       dwRetValue;

    TRC_0ENTER ("DPC_Callback") ;
    for (;;)
    {
        dwRetValue = WaitForSingleObject(dpcObj->hDpcEvent, INFINITE);
        if (InterlockedExchangeAdd(&dpcObj->exit,0)==1)
        {
            break;
        }
        if (dwRetValue != WAIT_OBJECT_0)
        {
            continue;
        }
        // Use InterlockedExchangeAdd with an increment of zero to get back the value of
        // the variable in an atomic way.
        while ( InterlockedExchangeAdd(&dpcObj->count, 0) && DSP_SUCCEEDED (status)) 
        {
            if (*dpcObj->proc!= NULL) 
            {
                // Call the user defined function, passing the parameters
                (*dpcObj->proc) (dpcObj->param) ;
                // Decrement the pending count
                InterlockedDecrement( &dpcObj->count );
            }
            else
            {
                status = DSP_EPOINTER ;
                SET_FAILURE_REASON ;
            }
        }
    }

    TRC_0LEAVE ("DPC_Callback") ;
    return 0;

}


#if defined (__cplusplus)
}
#endif
