/** ============================================================================
 *  @file   reg.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/WinCE/
 *
 *  @desc   Implementation of sub-component REG.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <windows.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- Generic Functions           */
#include <gen_utils.h>

#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and sub-component Identifier.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_OSAL_REG

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  TRC_SetReason (status, FID_C_OSAL_REG, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


#define MAXREGPATHLENGTH 255


/** ============================================================================
 *  @func   Reg_Initialize
 *
 *  @desc   This function initializes the REG module. 
 *
 *  @modif  
 *  ============================================================================
 */

Void Reg_Initialize(Void)
{
    /* do nothing */
}

/** ============================================================================
 *  @func   Reg_GetKeyValue
 *
 *  @desc   This function initializes the REG module. 
 *
 *  @modif  
 *  ============================================================================
 */

DSP_STATUS Reg_GetKeyValue(IN HKEY hKey, IN Pstr subkey, IN Pstr valueName,
                        OUT Uint8 *pData, IN OUT Int32 *pDataSize)
{
    LONG            lResult = 0L;
    DWORD           dwType = 0L;
    WCHAR           wszSubkey[MAXREGPATHLENGTH];
    WCHAR           wszValue[MAXREGPATHLENGTH];
    size_t          convertedChars=0;        
    Uint8          *pRegData;
    DSP_STATUS      status = DSP_SOK;
    HKEY            hKeyHandle;

    DBC_Require(strlen(subkey) <= MAXREGPATHLENGTH); 
    DBC_Require(strlen(valueName) <= MAXREGPATHLENGTH);
    DBC_Require(pData!= NULL);
    DBC_Require(pDataSize != NULL);
    
    /* Convert ANSI strings to UNICODE */
    if ((strlen(subkey) > MAXREGPATHLENGTH) ||
        (strlen(valueName) > MAXREGPATHLENGTH) ||
        (pDataSize == NULL) ||
        (pData == NULL)){        
            status = DSP_EINVALIDARG;
            SET_FAILURE_REASON;
    }   

    if (DSP_SUCCEEDED(status)) {
        /* Convert ANSI strings to UNICODE */
        mbstowcs_s (&convertedChars, wszSubkey, (MAXREGPATHLENGTH - 1) , subkey, _TRUNCATE);
        if(convertedChars == 0) {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }    
    }
    if (DSP_SUCCEEDED(status)) {
        convertedChars=0;  
        /* Convert ANSI strings to UNICODE */
        mbstowcs_s (&convertedChars, wszValue, (MAXREGPATHLENGTH - 1) , valueName, _TRUNCATE);
        if(convertedChars == 0) {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }    
    }
    
    if (DSP_SUCCEEDED(status)) {
        /* Open the registry subkey */
        lResult = RegOpenKeyEx(hKey, wszSubkey, 0, 0, &hKeyHandle);

        if (lResult != ERROR_SUCCESS) {
            status = DSP_EFAIL;
            SET_FAILURE_REASON ;
        }
        else {
            /* Allocate memory for query buffer. */
            status = MEM_Alloc(&pRegData,(*pDataSize), MEM_DEFAULT);
            if (DSP_SUCCEEDED(status)){
                /* Query registry value. */
                lResult = RegQueryValueEx(hKeyHandle, wszValue, NULL, &dwType,
                        pRegData, pDataSize);

                if (lResult == ERROR_SUCCESS) {
                    if ((dwType == REG_SZ) || (dwType == REG_MULTI_SZ) ||
                            (dwType == REG_EXPAND_SZ)) {
                        /* Convert UNICODE strings to ANSI. */
                        wcstombs_s(&convertedChars,pData,*pDataSize,(WCHAR *)pRegData,*pDataSize);
                    }
                    else {
                        memcpy(pData, pRegData, *pDataSize);
                    }
                }
                else if (lResult == ERROR_MORE_DATA) {
                    /* Data didn't fit */
                    status = DSP_ERANGE;
                    SET_FAILURE_REASON;
                }
                else {
                    status = DSP_EFAIL;
                    SET_FAILURE_REASON;
                }
                    
                /* Free allocated memory. */
                MEM_Free(&pRegData, MEM_DEFAULT);
            }
                
            /* close opened key */
            lResult = RegCloseKey (hKeyHandle);
        }
    }
    
    return (status);
    
}



