/** ============================================================================
 *  @file   message_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/message_multidrv/WinCE/
 *
 *  @desc   OS specific implementation of functions used by the message_multi
 *          sample application.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers           */
#include <windows.h>

/*  ----------------------------------- DSP/BIOS Link                 */
#include <dsplink.h>

/*  ----------------------------------- Application Header            */
#include <message.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

/** ============================================================================
 *  @name   MAXSTRLENGTH
 *
 *  @desc   Maximum Length of the Trace String.
 *  ============================================================================
 */
#define MAXSTRLENGTH        256


/** ============================================================================
 *  @name   MESSAGE_SemObject
 *
 *  @desc   This object is used by various SEM API's.
 *
 *  @field  sem
 *              WinCE semaphore handle.
 *
 *  @see    None
 *  ============================================================================
 */
typedef struct MESSAGE_SemObject_tag {
    HANDLE  sem ;
} MESSAGE_SemObject ;


/** ============================================================================
 *  @func   MESSAGEMULTI_OS_init
 *
 *  @desc   This function does OS specific initialization.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MESSAGEMULTI_OS_init (Void)
{
    DSP_STATUS          status = DSP_SOK ;

    return status ;
}


/** ============================================================================
 *  @func   MESSAGEMULTI_OS_exit
 *
 *  @desc   This function OS specific finalization.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MESSAGEMULTI_OS_exit(Void)
{
    DSP_STATUS status = DSP_SOK ;

    return status ;
}


/** ============================================================================
 *  @func   MESSAGEMULTI_0Print
 *
 *  @desc   Print a message without any arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTI_0Print (Char8 * str)
{
    TCHAR  buffer[MAXSTRLENGTH];    
    mbstowcs (buffer, (const char *)str, MAXSTRLENGTH);
    RETAILMSG(1, (TEXT("%s"), buffer));
}


/** ============================================================================
 *  @func   MESSAGEMULTI_1Print
 *
 *  @desc   Print a message with one arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTI_1Print (Char8 * str, Uint32 arg)
{
    TCHAR  buffer[MAXSTRLENGTH];
    TCHAR  fmtSpec[MAXSTRLENGTH];
    
    mbstowcs (fmtSpec, (const char *)str, MAXSTRLENGTH);
    wsprintf (buffer, fmtSpec, arg);
    RETAILMSG(1, (TEXT("%s"), buffer));
}


/** ============================================================================
 *  @func   MESSAGEMULTI_2Print
 *
 *  @desc   Print a message with two arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTI_2Print (Char8 * str, Uint32 arg1, Uint32 arg2)
{
    TCHAR  buffer[MAXSTRLENGTH];
    TCHAR  fmtSpec[MAXSTRLENGTH];
    
    mbstowcs (fmtSpec, (const char *)str, MAXSTRLENGTH);
    wsprintf (buffer, fmtSpec, arg1, arg2);
    RETAILMSG(1, (TEXT("%s"), buffer));
}


/** ============================================================================
 *  @func   MESSAGEMULTI_2PrintStr
 *
 *  @desc   Print a message with two arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTI_2PrintStr (Char8 * str, Uint32 arg1, Pstr arg2)
{
    TCHAR  buffer[MAXSTRLENGTH];
    TCHAR  fmtSpec[MAXSTRLENGTH];
    
    mbstowcs (fmtSpec, (const char *)str, MAXSTRLENGTH);
    wsprintf (buffer, fmtSpec, arg1, arg2);
    RETAILMSG(1, (TEXT("%s"), buffer));
}


/** ============================================================================
 *  @func   MESSAGEMULTI_Sleep
 *
 *  @desc   Sleeps for the specified number of microseconds.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTI_Sleep (Uint32 uSec)
{
    Sleep ((uSec + 999)/1000) ;    
}


/** ============================================================================
 *  @func   MESSAGEMULTI_CreateSem
 *
 *  @desc   This function creates a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MESSAGEMULTI_CreateSem (OUT Pvoid * semPtr)
{
    DSP_STATUS          status = DSP_SOK ;
    MESSAGE_SemObject * semObj ;
    int                 osStatus ;

    semObj = malloc (sizeof (MESSAGE_SemObject)) ;
    if (semObj != NULL) {
        semObj->sem = CreateSemaphore(NULL,0,1,NULL); // binary semaphore            
        if (semObj->sem == NULL) {
            status = DSP_EFAIL ;
        }
        else {
            *semPtr = (Pvoid) semObj ;
        }
    }
    else {
        *semPtr = NULL ;
        status = DSP_EFAIL ;
    }

    return status ;
}


/** ============================================================================
 *  @func   MESSAGEMULTI_DeleteSem
 *
 *  @desc   This function deletes a semaphore.
 *
 *  @arg    semHandle
 *              Pointer to the semaphore object to be deleted.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MESSAGEMULTI_DeleteSem (IN Pvoid semHandle)
{
    DSP_STATUS          status = DSP_SOK ;
    MESSAGE_SemObject * semObj = semHandle ;
    int                 osStatus ;

    osStatus = CloseHandle(semObj->sem) ;
    if (osStatus == 0) {
        status = DSP_EFAIL ;
    }

    free (semObj) ;

    return status ;
}


/** ============================================================================
 *  @func   MESSAGEMULTI_WaitSem
 *
 *  @desc   This function waits on a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MESSAGEMULTI_WaitSem (IN Pvoid semHandle)
{
    DSP_STATUS          status = DSP_SOK ;
    MESSAGE_SemObject * semObj = semHandle ;
    int                 osStatus ;

    osStatus = WaitForSingleObject(semObj->sem, 0xFFFFFFFF) ;//timeout=infinite
    if (osStatus != WAIT_OBJECT_0) {
        status = DSP_EFAIL ;
    }

    return status ;
}


/** ============================================================================
 *  @func   MESSAGEMULTI_PostSem
 *
 *  @desc   This function posts a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
MESSAGEMULTI_PostSem (IN Pvoid semHandle)
{
    DSP_STATUS          status = DSP_SOK ;
    MESSAGE_SemObject * semObj = semHandle ;
    int                 osStatus ;

    osStatus = ReleaseSemaphore(semObj->sem, 1, NULL) ;
    if (osStatus == 0) {
        status = DSP_EFAIL ;
    }

    return status ;
}



#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
