/** ============================================================================
 *  @file   readwrite_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/readwrite/PrOS/
 *
 *  @desc   OS specific implementation of functions used by the readwrite
 *          sample application.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers           */
#include <itron.h>

/*  ----------------------------------- DSP/BIOS Link                 */
#include <dsplink.h>

/*  ----------------------------------- Application Header            */
#include <readwrite.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

/** ============================================================================
 *  @name   MEM_mplId_sample
 *
 *  @desc   ID of memory pool created.
 *  ============================================================================
 */
STATIC ID MEM_mplId_sample ;

/** ============================================================================
 *  @macro  MPL_BUFF_SIZE
 *
 *  @desc   Memory pool size.
 *  ============================================================================
 */
#define  MPL_BUFF_SIZE  0x200000u

/** ============================================================================
 *  @name   mplbuff_sample_readwrite
 *
 *  @desc   Start address of the variable-sized memory pool area.
 *  ============================================================================
 */
Uint8 mplbuff_sample_readwrite [MPL_BUFF_SIZE] ;

/** ============================================================================
 *  @func   kprintf
 *
 *  @desc   Extern declaration for kprintf function.
 *
 *  @arg    str
 *              String representation of the number.
 *  @arg    ...
 *              Variable arguments.
 *
 *  @ret    None
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    None
 *  ============================================================================
 */
#if !defined (DRX45X)
extern void kprintf (char* fp, ... ) ;
#endif /* !defined (DRX45X) */


/** ============================================================================
 *  @func   RDWR_OS_init
 *
 *  @desc   This function creates a mem pool.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RDWR_OS_init()
{
    DSP_STATUS          status = DSP_SOK ;
    T_CMPL     pk_cmpl           ;
    ER_ID      mplId             ;

    pk_cmpl.mplatr = TA_TPRI ;
    pk_cmpl.mplsz = MPL_BUFF_SIZE ;
    pk_cmpl.mpl = (VP) mplbuff_sample_readwrite ;
    mplId = acre_mpl (&pk_cmpl) ;

    if (mplId > 0) {
        MEM_mplId_sample = mplId ;
    }
    else {
        status = DSP_EFAIL ;
    }


    return status ;
}


/** ============================================================================
 *  @func   RDWR_OS_exit
 *
 *  @desc   This function deletes a mem pool.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RDWR_OS_exit()
{
    DSP_STATUS status = DSP_SOK ;
    ER_ID      erStatus           ;

    erStatus = del_mpl (MEM_mplId_sample) ;

    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }
    else {
        MEM_mplId_sample = -1 ;
    }
    return status ;
}


/** ============================================================================
 *  @func   RDWR_0Print
 *
 *  @desc   Print a message without any arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RDWR_0Print (Char8 * str)
{
#if !defined (DRX45X)
   kprintf (str) ;
#else /* defined (DRX45X) */
   printf (str) ;
#endif /* !defined (DRX45X) */
}


/** ============================================================================
 *  @func   RDWR_1Print
 *
 *  @desc   Print a message with one arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RDWR_1Print (Char8 * str, Uint32 arg)
{
#if !defined (DRX45X)
   kprintf (str, arg) ;
#else /* defined (DRX45X) */
   printf (str, arg) ;
#endif /* !defined (DRX45X) */
}


/** ============================================================================
 *  @func   RDWR_Sleep
 *
 *  @desc   Sleeps for the specified number of microseconds.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RDWR_Sleep (Uint32 uSec)
{
    Uint32 waitMsecs = 1 ;

    /* Convert the usec into miliseconds. */
    if ((uSec / 1000) > 1) {
        waitMsecs = uSec / 1000 ;
    }

    dly_tsk((TMO)waitMsecs) ;
}


/** ============================================================================
 *  @func   RDWR_AllocateBuffer
 *
 *  @desc   Allocates a buffer of specified size.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RDWR_AllocateBuffer (IN Uint32 size, OUT Pvoid * buf)
{
    DSP_STATUS status = DSP_SOK ;

    if (DSP_SUCCEEDED (status)) {
        get_mpl (MEM_mplId_sample, size, (VP *) buf) ;

        if (*buf == NULL) {
            status = DSP_EMEMORY ;
        }
    }

    return status ;
}


/** ============================================================================
 *  @func   RDWR_FreeBuffer
 *
 *  @desc   Frees the specified buffer
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RDWR_FreeBuffer (IN OUT Pvoid * buf)
{
    rel_mpl (MEM_mplId_sample, (VP) *buf) ; ;
    *buf = NULL ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
