/** ============================================================================
 *  @file   ring_io_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/ring_io/PrOS/
 *
 *  @desc   OS specific implementation of functions used by the ring_io
 *          sample application.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */

/*  ----------------------------------- OS Specific Headers           */
#include <itron.h>
#include <string.h>
/*  ----------------------------------- DSP/BIOS Link                 */
#include <dsplink.h>

/*  ----------------------------------- Application Header            */
#include <ring_io.h>
#include <ring_io_os.h>



#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @const  SEM_NON_SIGNALLED
 *
 *  @desc   Non signalled semaphore value.
 *  ============================================================================
 */
#define SEM_NON_SIGNALLED    0u

/** ============================================================================
 *  @const  SEM_SIGNALLED
 *
 *  @desc   Signalled semaphore value.
 *  ============================================================================
 */
#define SEM_SIGNALLED        1u

/** ============================================================================
 *  @const  SEM_BINARY
 *
 *  @desc   Maximum count for binary type semaphore.
 *  ============================================================================
 */
#define SEM_BINARY           1u

/** ============================================================================
 *  @const  SEM_COUNTING
 *
 *  @desc   Maximum count for counting type semaphore.
 *  ============================================================================
 */
#define SEM_COUNTING         0xFFFFu

/** ============================================================================
 *  @name   RING_IO_MEM_mplId
 *
 *  @desc   ID of memory pool created.
 *  ============================================================================
 */
STATIC ID RING_IO_MEM_mplId ;

/** ============================================================================
 *  @macro  RING_IO_MPL_BUFF_SIZE
 *
 *  @desc   Memory pool size.
 *  ============================================================================
 */
#define  RING_IO_MPL_BUFF_SIZE  0x400000u

/** ============================================================================
 *  @name   RING_IO_mplbuff
 *
 *  @desc   Start address of the variable-sized memory pool area.
 *  ============================================================================
 */
Uint8 RING_IO_mplbuff [RING_IO_MPL_BUFF_SIZE] ;

/** ============================================================================
 *  @name   RING_IO_SemObject
 *
 *  @desc   This object is used by various SEM API's.
 *
 *  @field  semId
 *              ID number of semaphore.
 *
 *  @see    None
 *  ============================================================================
 */
typedef struct RING_IO_SemObject_tag {
    ID  semId ;
} RING_IO_SemObject ;


/** ============================================================================
 *  @func   kprintf
 *
 *  @desc   Extern declaration for kprintf function.
 *
 *  @arg    str
 *              String representation of the number.
 *  @arg    ...
 *              Variable arguments.
 *
 *  @ret    None
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    None
 *  ============================================================================
 */
#if !defined (DRX45X)
extern void kprintf (char* fp, ... ) ;
#endif /* !defined (DRX45X) */

/** ============================================================================
 *  @func   RING_IO_0Print
 *
 *  @desc   Print a message without any arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_0Print (Char8 * str)
{
#if !defined (DRX45X)
   kprintf (str) ;
#else /* defined (DRX45X) */
   printf (str) ;
#endif /* !defined (DRX45X) */
}

/** ============================================================================
 *  @func   RING_IO_1Print
 *
 *  @desc   Print a message with one arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_1Print (Char8 * str, Uint32 arg)
{
#if !defined (DRX45X)
   kprintf (str, arg) ;
#else /* defined (DRX45X) */
   printf (str, arg) ;
#endif /* !defined (DRX45X) */
}
/** ============================================================================
 *  @func   RING_IO_YieldTsk
 *
 *  @desc   Yields tasks execution.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_YieldClient ()
{
    dly_tsk(0) ;
}

/** ============================================================================
 *  @func   RING_IO_Sleep
 *
 *  @desc   Sleeps for the specified number of microseconds.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_Sleep (Uint32 uSec)
{
    Uint32 waitMsecs = 1 ;

    /* Convert the usec into miliseconds. */
    if ((uSec / 1000) > 1) {
        waitMsecs = uSec / 1000 ;
    }

    dly_tsk((TMO)waitMsecs) ;
}

/** ============================================================================
 *  @func   RING_IO_IntToString
 *
 *  @desc   This function converts an integer value into a string.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_IntToString (IN Int num, OUT Char8 * str)
{
    /*  ------------------------------------------------------------------------
     *  There can be a max of 7 char str for signed int including terminator
     *  so we use a temp string of 7 char. The index is to the rightmost space
     *  ie. 5. We divide the number by 10, and successively store the remainder
     *  in the rightmost space while decrementing the counter. Later we add the
     *  sign if required.
     *  ------------------------------------------------------------------------
     */
    Int16      sign      = 0 ;
    Char8      remainder = 0 ;
    char       temp [11] = {' ',' ',' ',' ',' ',' ',' ',' ',' ',' ','\0'} ;
    Int16      index     = 9 ;

    sign = (num < 0) ? -1 : 1 ;
    num = num * sign ;

    do {
        remainder = num % 10 ;
        temp [index] = '0' + remainder ;
        num = num / 10 ;
        index-- ;
    } while ((num > 0)&& (index >= 0)) ;


    if (sign == -1) {
        temp [index] = '-' ;
    }
    else {
        index++ ;
    }

    strncpy (str, &(temp [index]), 11) ;
}

/** ============================================================================
 *  @func   RING_IO_OS_init
 *
 *  @desc   This function creates a mem pool.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_OS_init()
{
    DSP_STATUS          status = DSP_SOK ;
    T_CMPL     pk_cmpl           ;
    ER_ID      mplId             ;

    pk_cmpl.mplatr = TA_TPRI ;
    pk_cmpl.mplsz = RING_IO_MPL_BUFF_SIZE ;
    pk_cmpl.mpl = (VP) RING_IO_mplbuff ;
    mplId = acre_mpl (&pk_cmpl) ;

    if (mplId > 0) {
        RING_IO_MEM_mplId = mplId ;
    }
    else {
        status = DSP_EFAIL ;
    }


    return status ;
}

/** ============================================================================
 *  @func   RING_IO_OS_exit
 *
 *  @desc   This function deletes a mem pool.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_OS_exit()
{
    DSP_STATUS status = DSP_SOK ;
    ER_ID      erStatus           ;

    erStatus = del_mpl (RING_IO_MEM_mplId) ;

    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }
    else {
        RING_IO_MEM_mplId = -1 ;
    }
    return status ;
}

/** ============================================================================
 *  @func   RING_IO_CreateSem
 *
 *  @desc   This function creates a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_CreateSem (OUT Pvoid * semPtr)
{
    DSP_STATUS          status = DSP_SOK ;
    RING_IO_SemObject * semObj = NULL ;
    ER_ID               erId   = E_OK ;
    T_CSEM              pk_csem ;

    get_mpl (RING_IO_MEM_mplId, sizeof (RING_IO_SemObject), (VP *) &semObj) ;
    if (semObj != NULL) {
        pk_csem.sematr  = TA_TFIFO ;
        pk_csem.isemcnt = (UINT) SEM_NON_SIGNALLED   ;
        pk_csem.maxsem  = (UINT) SEM_COUNTING ;
        erId = acre_sem (&pk_csem) ;
        if (erId <= 0) {
            rel_mpl (RING_IO_MEM_mplId, (VP) semObj) ;
            status = DSP_EFAIL ;
        }
        else {
            /* erId contains a valid semaphore id. */
            semObj->semId = erId ;
            *semPtr = (Pvoid) semObj ;
        }
    }
    else {
        *semPtr = NULL ;
        status = DSP_EMEMORY ;
    }

    return status ;
}

/** ============================================================================
 *  @func   RING_IO_DeleteSem
 *
 *  @desc   This function deletes a semaphore.
 *
 *  @arg    semHandle
 *              Pointer to the semaphore object to be deleted.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_DeleteSem (IN Pvoid semHandle)
{
    DSP_STATUS          status   = DSP_SOK ;
    RING_IO_SemObject * semObj   = semHandle ;
    ER                  erStatus = E_OK ;

    erStatus = del_sem (semObj->semId) ;
    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }

    rel_mpl (RING_IO_MEM_mplId, (VP) semObj) ;

    return status ;
}

/** ============================================================================
 *  @func   RING_IO_WaitSem
 *
 *  @desc   This function waits on a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_WaitSem (IN Pvoid semHandle)
{
    DSP_STATUS          status   = DSP_SOK ;
    RING_IO_SemObject * semObj   = semHandle ;
    ER                  erStatus = E_OK ;

    erStatus = wai_sem (semObj->semId) ;
    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }

    return status ;
}

/** ============================================================================
 *  @func   RING_IO_PostSem
 *
 *  @desc   This function posts a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_PostSem (IN Pvoid semHandle)
{
    DSP_STATUS          status   = DSP_SOK ;
    RING_IO_SemObject * semObj   = semHandle ;
    ER                  erStatus = E_OK ;
    BOOL                state ;

    /* get the current context. */
    state = sns_ctx () ;

    if (state == FALSE) {
        erStatus = sig_sem (semObj->semId) ;
    }
    else {
        erStatus = isig_sem (semObj->semId) ;
    }

    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }

    return status ;
}

/** ============================================================================
 *  @func   RING_IO_Create_client
 *
 *  @desc   Function to create new thread.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Uint32
RING_IO_Create_client (RING_IO_ClientInfo * pInfo, Pvoid funcPtr, Pvoid args)
{
    DSP_STATUS status   = DSP_SOK ;
    ER_ID      erId     = E_OK    ;
    ER         erStatus = E_OK    ;
    T_CTSK     pk_ctsk            ;
    T_RTSK     pk_rtsk            ;

    erStatus = ref_tsk (TSK_SELF, &pk_rtsk);

    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }

    pk_ctsk.tskatr  = TA_ACT |TA_HLNG         ;
    pk_ctsk.exinf   = (VP_INT) args   ;
    pk_ctsk.task    = (FP) ((int)funcPtr)    ;

    pk_ctsk.itskpri =  22/*pk_rtsk.tskbpri */;

    /* stksz  */
    pk_ctsk.stksz = 0x1024 ;

    /* Allocate stack memory. */
    erId = tget_mpl(RING_IO_MEM_mplId, pk_ctsk.stksz, &pk_ctsk.stk, 0x100);

    if (erId != E_OK) {
        status = DSP_EMEMORY ;
    }
    else if (erId == E_TMOUT) {
        RING_IO_0Print ("Thread Create: Unable to allocate memory for stack\n") ;
        RING_IO_0Print ("Increase the default MPL memory") ;
    }
    else {
        erId = acre_tsk (&pk_ctsk);

        if (erId <= 0) {
            status = DSP_EFAIL ;
            pInfo->tid = (ID) -1 ;
        }
        else {
            pInfo->tid  = erId    ;
            pInfo->stk  = pk_ctsk.stk ;
        }
    }

    return status ;
}

/** ============================================================================
 *  @func   RING_IO_Join_client
 *
 *  @desc   Wait for thread to exit.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
RING_IO_Join_client (RING_IO_ClientInfo * pInfo)
{
    DSP_STATUS status   = DSP_SOK ;
    ER         erStatus = E_OK    ;
    Bool       dmtState = FALSE   ;
    T_RTSK     pk_rtsk            ;

    while ((DSP_SUCCEEDED (status)) && (dmtState == FALSE)) {

        /* Get the thread info */
        erStatus = ref_tsk (pInfo->tid, &pk_rtsk);

        if (erStatus != E_OK) {
            status = DSP_EFAIL ;
        }
        else {
            /* Check if the thread goes into Dormant state. i.e stopped. */
            if (pk_rtsk.tskstat == TTS_DMT) {
                dmtState = TRUE ;
            }
            else {
                /* Wait for the create task to finish. */
                /* Not Required */
                dly_tsk (0x10) ;
            }
        }

    }

    return (status) ;
}

/** ============================================================================
 *  @func   RING_IO_Exit_client
 *
 *  @desc   Function to exit from a caller thread. The pointer structure passed
 *          should  consists   caller thread info .
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
RING_IO_Exit_client (RING_IO_ClientInfo * pInfo)
{
    DSP_STATUS status   = DSP_SOK ;
    ER         erStatus = E_OK    ;

    erStatus = del_tsk ((ID) pInfo->tid);

    rel_mpl (RING_IO_MEM_mplId, pInfo->stk) ;

    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
    }

    return status ;
}
#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
