/** ============================================================================
 *  @file   ring_io_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/ring_io/WinCE/
 *
 *  @desc   OS specific implementation of functions used by the ring_io
 *          sample application.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers           */
#include <windows.h>

/*  ----------------------------------- DSP/BIOS Link                 */
#include <dsplink.h>
#include <proc.h>
/*  ----------------------------------- Application Header            */
#include <ring_io.h>
#include <ring_io_os.h>

#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @name   RING_IO_SemObject
 *
 *  @desc   This object is used by various SEM API's.
 *
 *  @field  sem
 *              Linux semaphore.
 *
 *  @see    None
 *  ============================================================================
 */
typedef struct RING_IO_SemObject_tag {
    HANDLE  sem ;
} RING_IO_SemObject ;

/** ============================================================================
 *  @name   MAXSTRLENGTH
 *
 *  @desc   Maximum Length of the Trace String.
 *  ============================================================================
 */
#define MAXSTRLENGTH        256

/** ============================================================================
 *  @func   RING_IO_0Print
 *
 *  @desc   Print a message without any arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_0Print (Char8 * str)
{
    TCHAR  buffer[MAXSTRLENGTH];    
    mbstowcs (buffer, (const char *)str, MAXSTRLENGTH);
    RETAILMSG(1, (TEXT("%s"), buffer));
}

/** ============================================================================
 *  @func   RING_IO_1Print
 *
 *  @desc   Print a message with one arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_1Print (Char8 * str, Uint32 arg)
{
    TCHAR  buffer[MAXSTRLENGTH];
    TCHAR  fmtSpec[MAXSTRLENGTH];
    
    mbstowcs (fmtSpec, (const char *)str, MAXSTRLENGTH);
    wsprintf (buffer, fmtSpec, arg);
    RETAILMSG(1, (TEXT("%s"), buffer));  
}
/** ============================================================================
 *  @func   RING_IO_YieldClient
 *
 *  @desc   Yields tasks execution.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_YieldClient ()
{
    Sleep(0) ;
}
/** ============================================================================
 *  @func   RING_IO_Sleep
 *
 *  @desc   Sleeps for the specified number of microseconds.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_Sleep (Uint32 uSec)
{
    Sleep ((uSec + 999)/1000) ;    
}

/** ============================================================================
 *  @func   RING_IO_IntToString
 *
 *  @desc   This function converts an integer value into a string.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
RING_IO_IntToString (IN Int num, OUT Char8 * str)
{
    _itoa( num, str, 10 );
}

/** ============================================================================
 *  @func   RING_IO_CreateSem
 *
 *  @desc   This function creates a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_CreateSem (OUT Pvoid * semPtr)
{
    DSP_STATUS          status = DSP_SOK ;
    RING_IO_SemObject * semObj ;
    int                 osStatus ;

    semObj = malloc (sizeof (RING_IO_SemObject)) ;
    if (semObj != NULL) {
        semObj->sem = CreateSemaphore(NULL,0,1,NULL); // binary semaphore            
        if (semObj->sem == NULL) {
            status = DSP_EFAIL ;
        }
        else {
            *semPtr = (Pvoid) semObj ;
        }
    }
    else {
        *semPtr = NULL ;
        status = DSP_EFAIL ;
    }

    return (status) ;
}

/** ============================================================================
 *  @func   RING_IO_DeleteSem
 *
 *  @desc   This function deletes a semaphore.
 *
 *  @arg    semHandle
 *              Pointer to the semaphore object to be deleted.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_DeleteSem (IN Pvoid semHandle)
{
    DSP_STATUS          status = DSP_SOK ;
    RING_IO_SemObject * semObj = semHandle ;
    int                 osStatus ;

    osStatus = CloseHandle(semObj->sem) ;
    if (osStatus == 0) {
        status = DSP_EFAIL ;
    }

    free (semObj) ;

    return (status) ;
}

/** ============================================================================
 *  @func   RING_IO_WaitSem
 *
 *  @desc   This function waits on a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_WaitSem (IN Pvoid semHandle)
{
    DSP_STATUS          status = DSP_SOK ;
    RING_IO_SemObject * semObj = semHandle ;
    int                 osStatus ;

    osStatus = WaitForSingleObject(semObj->sem, 0xFFFFFFFF) ;//timeout=infinite
    if (osStatus != WAIT_OBJECT_0) {
        status = DSP_EFAIL ;
    }

    return (status) ;
}

/** ============================================================================
 *  @func   RING_IO_PostSem
 *
 *  @desc   This function posts a semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_PostSem (IN Pvoid semHandle)
{
    DSP_STATUS          status = DSP_SOK ;
    RING_IO_SemObject * semObj = semHandle ;
    int                 osStatus ;

    osStatus = ReleaseSemaphore(semObj->sem, 1, NULL) ;
    if (osStatus == 0) {
        status = DSP_EFAIL ;
    }

    return (status) ;
}

#ifdef RING_IO_MULTIPROCESS
 /** ============================================================================
 *  @func   RING_IO_getLinkAccess
 *
 *  @desc   Function  that allows the child  process to  use the link
 *          components.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Int
RING_IO_getLinkAccess(Uint8 processorId)
{
    DSP_STATUS  status   = DSP_SOK ;

    /* Call the Link APIs that allows child process to be able to use the
     * Link components
     */

    status = PROC_Attach (processorId, NULL) ;

    if (DSP_FAILED (status)) {
        RING_IO_1Print ("Attach Failed. Status = [0x%x]", status) ;
    }

    return (status) ;
}
#endif

/** ============================================================================
 *  @func   RING_IO_Create_client
 *
 *  @desc   Function to create a new thread or a Process.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Uint32
RING_IO_Create_client (RING_IO_ClientInfo * pInfo, Pvoid funcPtr, Pvoid args)
{
  

#ifdef RING_IO_MULTIPROCESS
    /* should implement child process logic here */
#else

    pInfo->tid = CreateThread(NULL, 
                              0, 
                              funcPtr, 
                              args, 
                              0, 
                              NULL); 
    if (pInfo->tid == NULL)
        return (DSP_EFAIL);
    else
        return (DSP_SOK); //success
#endif  

    return (DSP_SOK) ;
}

/** ============================================================================
 *  @func   RING_IO_Join_client
 *
 *  @desc   Wait for thread/process to terminate.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
RING_IO_Join_client (RING_IO_ClientInfo *pInfo)
{
    DSP_STATUS  status   = DSP_SOK ;

#ifdef RING_IO_MULTIPROCESS
    /* do something */
#else
    WaitForSingleObject(pInfo->tid, INFINITE);
#endif
    
    return (status) ;
}

/** ============================================================================
 *  @func   RING_IO_Exit_client
 *
 *  @desc   Function to exit from a caller task(thread/process).
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
RING_IO_Exit_client (RING_IO_ClientInfo *pInfo)
{
    DSP_STATUS status   = DSP_SOK;

#ifdef RING_IO_MULTIPROCESS
    /* do something */
#else
    ExitThread(0);
#endif

    return (status) ;
}


/** ============================================================================
 *  @func   RING_IO_OS_init
 *
 *  @desc   This function creates a mem pool.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_OS_init(Void)
{
    DSP_STATUS  status = DSP_SOK ;

    return status ;
}


/** ============================================================================
 *  @func   RING_IO_OS_exit
 *
 *  @desc   This function deletes a mem pool.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
RING_IO_OS_exit(Void)
{
    DSP_STATUS status = DSP_SOK ;

    return status ;
}

#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
